
function test_momentum_rotor()
    D = rand()*180
    dt = rand()
    par = MomentumRotorParameters(diameter=D, time_step=dt)
    @test par.diameter == D
    @test par.time_step == dt
    @test par.radius == D/2
    @test par.area == π*(D/2)^2
    @test typeof(par) == MomentumRotorParameters

    a = rand()
    v = rand()
    w = rand()
    controls = MomentumRotorControls(induction=a, velocity=v, omega=w)
    @test controls.induction == a
    @test controls.velocity == v
    @test controls.omega == w
    @test typeof(controls) == MomentumRotorControls

    states = MomentumRotorStates(0.)
    rotor = MomentumRotor(par, controls, states)
    @test typeof(rotor) == MomentumRotor
    @test rotor.controls == controls
    @test rotor.parameters == par

    a_new = rand()
    v_new = rand()
    w_new = rand()
    # rotor.controls.induction = a_new
    # rotor.controls.velocity = v_new
    # rotor.controls.omega = w_new
    update_controls!(rotor, [a_new, v_new, w_new])
    @test rotor.controls.induction == a_new
    @test rotor.controls.velocity == v_new
    @test rotor.controls.omega == w_new

end
test_momentum_rotor()

function test_default_momentum_rotor()
    position =  @SVector randn(3)
    tower = construct_fixed_tower_10mw(position)

    rotor = construct_momentum_rotor_10mw()

    update_controls!(rotor,[0.33,SVector{3}([1,0,0]),6])
    ct = thrust_coefficient(rotor)
    cp = power_coefficient(rotor)
    T = calculate_thrust(rotor, tower)
    P = calculate_aerodynamic_power(rotor, tower)
end


test_momentum_rotor()
test_default_momentum_rotor()