

function test_generate_default_controls()
    num_steps = rand(10:1:100)
    @show m = generate_default_controls(num_steps)
    @test size(m) == (num_steps+1,)
end
test_generate_default_controls()

function test_generate_steady_inflow()
    speed = rand(0:.1:10.)
    direction = rand(-180:.1:180)
    u = generate_inflow(speed, direction)
    @test isapprox(norm(u), speed)
    @test isapprox(u[3], 0) 
    # atan returns on domain -π to π
    @test isapprox(direction, rad2deg(atan(-u[2],u[1])))

    u = generate_inflow(speed, 90.)
    @test isapprox([0,-1,0],u./norm(u))
    u = generate_inflow(speed, -90.)
    @test isapprox([0,1,0],u./norm(u))
end
test_generate_steady_inflow()

function test_run_forward()
    wake = construct_default_adm_wake()
    num_steps = rand(5:1:15)
    # num_steps = 100
    m = generate_default_controls(num_steps)
    m[end] = m[end] .+ .01
    u = generate_inflow(10., 0.)
    run_forward!(wake, m, u)
    plot_points(wake)
    @test wake.parameters.rotor.controls.induction == m[end][1]
    @test wake.parameters.tower.controls.yaw == m[end][2]

    @test all(isapprox.(u, wake.controls.inflow))
    wake = construct_default_admr_wake()
    # m = [SVector{2}([0.33, 30.]) for c in 1:100]
    run_forward!(wake, m, u)
    # plot_points(wake)
    @test wake.parameters.rotor.controls.induction == m[end][1]
    @test wake.parameters.tower.controls.yaw == m[end][2]
    # plot_points(wake)
    # @show wake.states.positions
    
    num_steps = rand(5:15)
    # num_steps = 150
    m = generate_default_controls(num_steps)
    m[end] = m[end] .+ .01

    u = generate_inflow(10., 0.)
    wake = construct_default_joukowsky_wake()
    # m = [SVector{2}([0.33, 30.]) for c in 1:600]
    # wake = construct_default_admr_wake()
    
    # m = [SVector{2}([0.33, 30.]) for c in 1:200]
    run_forward!(wake, m, u)
    plot_points(wake)
    @test wake.parameters.rotor.controls.induction == m[end][1]
    @test wake.parameters.tower.controls.yaw == m[end][2]
    plot_points(wake)

    @show uind = induced_velocity([SVector{3}([500.,0,119])],wake)
    # @show wake.states.positions
end
test_run_forward()

