include("mowitobase.jl")

function construct_adm_unit(time_step=0.3, num_elements=16, core_size=0.16, num_rings=40)
    rotor = construct_momentum_rotor_unit()
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = ADMWakeParameters(rotor=rotor, tower=tower,
        # num_rings=Int(floor(40*0.3/time_step)), num_elements=num_elements,
        num_rings=num_rings, num_elements=num_elements,
        rotor_points=rotor_disc_points() ,
        time_step=time_step,
         initial_core_size=core_size,delta=10.)
    states = initialise_states(params)
    controls = ADMWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)))
    return ADMWake(params, states, controls)
end

function get_wake_deficit(wake, xv, nsub=1)
    num_steps = Int(floor(30/wake.parameters.time_step))
    m = [SVector{2}([0.33, 0.]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    VortexWake.run_forward_substep!(wake,m,u, nsub)
    points = [SVector{3}([x, 0, 0]) for x in xv]
    points = VortexWake.transform(points, wake.parameters.tower)
    vel = induced_velocity(points, wake)
    pos = wake.states.positions
    return vel, pos
end
function get_wake_deficit_Heun(wake, xv, nsub=1)
    num_steps = Int(floor(30/wake.parameters.time_step))
    m = [SVector{2}([0.33, 0.]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    VortexWake.run_forward_Heun!(wake,m,u, nsub)
    points = [SVector{3}([x, 0, 0]) for x in xv]
    points = VortexWake.transform(points, wake.parameters.tower)
    vel = induced_velocity(points, wake)
    pos = wake.states.positions
    return vel, pos
end

function run_convergence_deficit(wake_constructor, h0, e0, s0, name)
    # wake_constructor = construct_adm_unit
    get_wake_deficit_mod(nsub) = get_wake_deficit(wake_constructor(h0, e0, s0), xv, nsub)
    # hs = 0.125:0.125:2
    xv = -2:0.1:7
    # hs = 0.1:0.1:2 #:0.25:2 #[0.5, 1, 1.5,2] #, 1.5]
    substeps = [1:1:10... 12:2:20... 24:4:40... 50:8:90... 200][:]
    result = map(get_wake_deficit_mod, substeps)
    vel = [r[1] for r in result]
    pos = [r[2] for r in result]

    columns = [@sprintf("h%04d",h) for h in substeps]
    data = [vel[a][b][1] for a in eachindex(vel), b in eachindex(vel[1])]
    push!(columns, "x")
    df = DataFrame([data' xv],columns)
    CSV.write(@sprintf("./data/convergence_deficit_%s_%05d.csv",name, Int(round(s0*1e5))), df)

    coords = ["x", "y", "z"]
    columns = [@sprintf("h%04d",h) for h in substeps]
    columns = [col*coord for coord in coords for col in columns ]
    [[p[1] for p in pts] for pts in pos]
    data_x = [pos[a][b][1] for  b in eachindex(pos[1]), a in eachindex(pos)]
    data_y = [pos[a][b][2] for  b in eachindex(pos[1]), a in eachindex(pos)]
    data_z = [pos[a][b][3] for  b in eachindex(pos[1]), a in eachindex(pos)]

    df = DataFrame([data_x data_y data_z],columns)
    CSV.write(@sprintf("./data/convergence_position_%s_%05d.csv",name, Int(round(s0*1e5))), df)

    return df
end
function run_convergence_deficit_Heun(wake_constructor, h0, e0, s0, name)
    # wake_constructor = construct_adm_unit
    get_wake_deficit_mod(nsub) = get_wake_deficit_Heun(wake_constructor(h0, e0, s0), xv, nsub)
    # hs = 0.125:0.125:2
    xv = -2:0.1:7
    # hs = 0.1:0.1:2 #:0.25:2 #[0.5, 1, 1.5,2] #, 1.5]
    # substeps = 1:1:20
    substeps = [1:1:10... 12:2:20... 24:4:40... 50:8:90... 200][:]
    result = map(get_wake_deficit_mod, substeps)
    vel = [r[1] for r in result]
    pos = [r[2] for r in result]

    columns = [@sprintf("h%04d",h) for h in substeps]
    data = [vel[a][b][1] for a in eachindex(vel), b in eachindex(vel[1])]
    push!(columns, "x")
    df = DataFrame([data' xv],columns)
    CSV.write(@sprintf("./data/convergence_deficit_Heun_%s_%05d.csv",name, Int(round(s0*1e5))), df)

    coords = ["x", "y", "z"]
    columns = [@sprintf("h%04d",h) for h in substeps]
    columns = [col*coord for coord in coords for col in columns ]
    [[p[1] for p in pts] for pts in pos]
    data_x = [pos[a][b][1] for  b in eachindex(pos[1]), a in eachindex(pos)]
    data_y = [pos[a][b][2] for  b in eachindex(pos[1]), a in eachindex(pos)]
    data_z = [pos[a][b][3] for  b in eachindex(pos[1]), a in eachindex(pos)]

    df = DataFrame([data_x data_y data_z],columns)
    CSV.write(@sprintf("./data/convergence_position_Heun_%s_%05d.csv",name, Int(round(s0*1e5))), df)

    return df
end

function get_cross_stream(wake, yv)
    num_steps = Int(floor(30/wake.parameters.time_step))
    m = [SVector{2}([0.33, 30.]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    run_forward!(wake,m,u)
    # run_forward_PC2B!(wake,m,u)
    # plot_points(wake)
    pts = VortexWake.transform(rotor_disc_points(), wake.parameters.tower).+ [[5., 0, 0]]
    # y = 0.
    vel = [sum(velocity(pts .+[[0.,y,0.]], wake))/length(pts) for y in yv]
    return vel
end

function run_convergence_deflection(wake_constructor, h0, e0, s0, name)
    yv = -2:0.05:2
    get_cross_stream_mod(e) = get_cross_stream(wake_constructor(h0, e, s0), yv)
    # ne =  4:4:80
    ne = [4:4:40... 48:8:80... 96:16:160... 256][:] #2 .* round.(4 .^(1:0.1:4) ./ 2)
    vel = map(get_cross_stream_mod, ne)
    columns = [@sprintf("e%03d",Int(e)) for e in ne]
    data = [vel[a][b][1] for a in eachindex(vel), b in eachindex(vel[1])]
    push!(columns, "y")
    df = DataFrame([data' yv],columns)
    CSV.write(@sprintf("./data/convergence_deflection_%s.csv",name), df)
    return df
end

function run_adm_convergence()
    for s in 0.04:0.04:0.16
        run_convergence_deficit(construct_adm_unit, 0.3, 16, s, "ADM_paper")
    end
    # run_convergence_deficit(construct_adm_unit, 0.3, 16, 0.08, "ADM")
    run_convergence_deflection(construct_adm_unit, 0.3, 16, 0.16, "ADM_paper")
end


function tube_deficit(wake, xv)
    X0 = VortexWake.initial_points(wake)
    X = wake.states.positions
    p = wake.parameters
    xp = range(start=0,step=p.time_step,length=p.num_rings)
    dX = [SVector{3}([x,0,0]) for a in [1], x in xp]
    X = broadcast(+,X,X0)
    X = broadcast(+,X,dX)
    
    thrust = calculate_thrust(wake.parameters.rotor, wake.parameters.)
    inflow = SVector{3}([1., 0, 0])
    update_controls!(wake, [thrust, inflow...])
    gamma = p.time_step * wake.controls.thrust / (wake.parameters.rotor.parameters.air_density * p.rotor.parameters.area)

    wake.states.positions = X
    wake.states.vortex_strengths .= gamma
    wake.states.core_sizes .= p.initial_core_size
    points = VortexWake.transform([SVector{3}([x, 0., 0]) for x in xv], p.tower)
    vel = induced_velocity(points, wake)
    # plot(scatter(x=xv,y=[v[1] for v in vel]))
    return vel
end
# plot(scatter(x=xv,y=[v[1] for v in vel]))
function run_spatial_convergence()
    name = "ADM_paper"
    h0 = 0.3
    e0 = 16
    s0 = 0.16
    r0 = 40
    xv = -2:0.1:7
    h = 0.5
    get_tube_deficit(h) = tube_deficit(construct_adm_unit(h, e0, s0, Int(round(r0*h0/h))), xv)
    # hvals = 0.001:0.01:0.5
    # hvals = [0.5]
    hvals = 0.5 .* 10 .^(0:-0.1:-3)
    vel = map(get_tube_deficit, hvals)
    data = [vel[a][b][1] for a in eachindex(vel), b in eachindex(vel[1])]
    columns =  [@sprintf("h%05d",Int(round(h*1e5))) for h in hvals]

    df = DataFrame(data',columns)
    CSV.write(@sprintf("./data/convergence_spatial_%s.csv",name), df)
end



h0 = 0.3
e0 = 16
s0 = 0.16

run_convergence_deficit(construct_adm_unit, h0, e0, s0, "ADM_paper")
run_convergence_deficit_Heun(construct_adm_unit, h0, e0, s0, "ADM_paper2")