using BenchmarkTools
using MAT
using Interpolations: linear_interpolation, Line

include("mowitobase.jl")

function load_wt_data()

    df = DataFrame(CSV.File("./ojf_yaw.csv"))

    diameter_wt = 0.58
    velocity_wt = 4.7
    wt_to_nd = velocity_wt / diameter_wt
    
    yaw = linear_interpolation(df.time.*wt_to_nd, df.yaw, extrapolation_bc =Line())
    wdir = linear_interpolation(df.time.*wt_to_nd, 0 .*df.yaw, extrapolation_bc =Line())
    return yaw, wdir
end

function run_forward_wt_exp!(wake, controls, inflow, winddir)
    pos_wt1_base = SVector{3}([1,0,0.])
    pos_wt2_base = SVector{3}([2,0,0.])
    pos_wt3_base = SVector{3}([3,0,0.])
    pos_wt4_base = SVector{3}([4,0,0.])


    pos_wt1_base = pos_wt1_base + [0,0,wake.parameters.tower.parameters.hubheight]
    pos_wt2_base = pos_wt2_base + [0,0,wake.parameters.tower.parameters.hubheight]
    pos_wt3_base = pos_wt3_base + [0,0,wake.parameters.tower.parameters.hubheight]
    pos_wt4_base = pos_wt4_base + [0,0,wake.parameters.tower.parameters.hubheight]

    omega = 6. * norm(inflow)/wake.parameters.rotor.parameters.radius
    columns = ["time", "ur0","ur1", "ur2","ur3","ur4","yaw", "wdir"]
    # data = zeros(length(columns), length(controls))
    rp = rotor_disc_points()

    np = length(rp)

    # for wt in 0:1, p in 1:1:np
    #     @show push!(columns, @sprintf("wt%d_u%d",wt,p), @sprintf("wt%d_v%d",wt,p), @sprintf("wt%d_w%d",wt,p) )
    # end
    data = zeros(length(columns), length(controls))

    for k in eachindex(controls)
        # @show controls[k]
        induction, yaw = controls[k]
        @show k
        u_rotor = rotor_velocity(wake)
        thrust = calculate_thrust(wake.parameters.rotor, wake.parameters.tower)
        wt0_pos = VortexWake.transform(rp, wake.parameters.tower)
        wt1_pos = rp .+ [pos_wt1_base]
        wt2_pos = rp .+ [pos_wt2_base]
        wt3_pos = rp .+ [pos_wt3_base]
        wt4_pos = rp .+ [pos_wt4_base]

        wt0_vel = velocity(wt0_pos, wake)
        wt1_vel = velocity(wt1_pos, wake)
        wt2_vel = velocity(wt2_pos, wake)
        wt3_vel = velocity(wt3_pos, wake)
        wt4_vel = velocity(wt4_pos, wake)
        data[1,k] = wake.parameters.time_step * k
        data[2,k] = u_rotor
        data[3,k] = sum(wt1_vel)[1]/length(rp)
        data[4,k] = sum(wt2_vel)[1]/length(rp)
        data[5,k] = sum(wt3_vel)[1]/length(rp)
        data[6,k] = sum(wt4_vel)[1]/length(rp)
        data[7,k] = yaw
        data[8,k] = winddir[k]
        # for p in 1:1:np
        #     data[5+3*(p-1) .+ (1:3), k] = wt0_vel[p]
        #     data[5+3*np+3*(p-1) .+ (1:3), k] = wt1_vel[p]

        # end
        # todo: find a better way to implement this, too easy to make mistakes here
        update_controls!(wake, [thrust, inflow..., omega])
        update_controls!(wake.parameters.rotor, [induction,u_rotor, omega] )
        update_controls!(wake.parameters.tower, [thrust, yaw] )

        step_forward!(wake)
        step_forward!(wake.parameters.rotor)
        step_forward!(wake.parameters.tower)
    end
    return DataFrame(data',columns)
end
function run_transient(wake)
    m = generate_default_controls(Int(24÷wake.parameters.time_step))
    u = generate_inflow(1., 0.)
    run_forward!(wake, m, u)
    return deepcopy(wake)
end
function run_experiment(wake, name)
    wake = run_transient(wake)
    yaw, wdir = load_wt_data()
    t = (0:wake.parameters.time_step:40)
    # time = (0:wake.parameters.time_step:300)
    controls = [SVector{2}([0.33, yaw(tv)]) for tv in t]
    inflow = generate_inflow(1., 0.)
    theta = wdir.(t)
    data =run_forward_wt_exp!(wake, controls, inflow, theta)
    CSV.write(@sprintf("./data/dynamictest_ojf_%s.csv",name), data)
    return wake
end
function run_adm_experiment()
    wake = construct_adm_wt_unit(40,16,10)
    wake = run_experiment(wake, "ADM")
    return wake
end
function run_admr_experiment()
    wake = construct_admr_wt_unit(40,16,10)
    run_experiment(wake, "ADMR")
end
function run_jouk_experiment()
    wake = construct_joukowsky_wt_unit(10)
    run_experiment(wake, "jouk")
end

wake = run_adm_experiment()
run_admr_experiment()
run_jouk_experiment()
