using BenchmarkTools
using MAT
using Interpolations: linear_interpolation

include("mowitobase.jl")

function load_wt_data(ctrlset="BW600")
     # CtrlNone
    # vars = matread("./data/FW_WindTunnelData.mat")["FW_WindTunnelData"]
    vars = matread("./data/FW_WindTunnelData_13012023.mat")["FW_WindTunnelData"]
    traverse = vars["Traverse"]
    wt0 = vars["Upstream_Turbine_Data"]["Ctrl"*ctrlset]
    wt1 = vars["Downstream_Turbine_Data"]["Ctrl"*ctrlset]

    wt0_yaw = vars["Yaw_upstr_out"][ctrlset]
    diameter_wt = 0.58
    velocity_wt = 7.3 #6.835
    wt_to_nd = velocity_wt / diameter_wt
    
    yaw = linear_interpolation(wt0_yaw[:,1].*wt_to_nd, wt0_yaw[:,2], extrapolation_bc = 0.)
    wdir = linear_interpolation(traverse[:,1].*wt_to_nd, traverse[:,2], extrapolation_bc = 0.)
    return yaw, wdir
end

function run_forward_wt_exp!(wake, controls, inflow, winddir)
    pos_wt1_base = SVector{3}([2.66,0,0.16])
    pos_wt1_base = pos_wt1_base + [0,0,wake.parameters.tower.parameters.hubheight]
    omega = 6. * norm(inflow)/wake.parameters.rotor.parameters.radius
    columns = ["time", "ur0","ur1","yaw", "wdir"]
    # data = zeros(length(columns), length(controls))
    rp = rotor_disc_points()

    np = length(rp)

    for wt in 0:1, p in 1:1:np
        @show push!(columns, @sprintf("wt%d_u%d",wt,p), @sprintf("wt%d_v%d",wt,p), @sprintf("wt%d_w%d",wt,p) )
    end
    data = zeros(length(columns), length(controls))

    for k in eachindex(controls)
        # @show controls[k]
        induction, yaw = controls[k]
        @show k
        u_rotor = rotor_velocity(wake)
        thrust = calculate_thrust(wake.parameters.rotor, wake.parameters.tower)
        wt0_pos = VortexWake.transform(rp, wake.parameters.tower)
        wt1_pos = rp .+ [VortexWake.rot_z(deg2rad(winddir[k])) * pos_wt1_base]
        wt0_vel = velocity(wt0_pos, wake)
        wt1_vel = velocity(wt1_pos, wake)
        data[1,k] = wake.parameters.time_step * k
        data[2,k] = u_rotor
        data[3,k] = sum(wt1_vel)[1]/length(rp)
        data[4,k] = yaw
        data[5,k] = winddir[k]
        for p in 1:1:np
            data[5+3*(p-1) .+ (1:3), k] = wt0_vel[p]
            data[5+3*np+3*(p-1) .+ (1:3), k] = wt1_vel[p]

        end
        # todo: find a better way to implement this, too easy to make mistakes here
        update_controls!(wake, [thrust, inflow..., omega])
        update_controls!(wake.parameters.rotor, [induction,u_rotor, omega] )
        update_controls!(wake.parameters.tower, [thrust, yaw] )

        step_forward!(wake)
        step_forward!(wake.parameters.rotor)
        step_forward!(wake.parameters.tower)
    end
    return DataFrame(data',columns)
end
function run_transient(wake)
    m = generate_default_controls(Int(24÷wake.parameters.time_step))
    u = generate_inflow(1., 0.)
    run_forward!(wake, m, u)
    return deepcopy(wake)
end
function run_experiment(wake, name, ctrlset)
    wake = run_transient(wake)
    yaw, wdir = load_wt_data(ctrlset)
    time = (0:wake.parameters.time_step:8000)
    # time = (0:wake.parameters.time_step:300)
    controls = [SVector{2}([0.33, yaw(t)]) for t in time]
    inflow = generate_inflow(1., 0.)
    theta = wdir.(time)
    data =run_forward_wt_exp!(wake, controls, inflow, theta)
    CSV.write(@sprintf("./data/dynamictest_%s_%s.csv",name,ctrlset), data)
end
function run_adm_experiment(ctrlset)
    wake = construct_adm_wt_unit(40,16,10)
    run_experiment(wake, "ADM", ctrlset)
end
function run_admr_experiment(ctrlset)
    wake = construct_admr_wt_unit(40,16,10)
    run_experiment(wake, "ADMR", ctrlset)
end
function run_jouk_experiment(ctrlset)
    wake = construct_joukowsky_wt_unit(10)
    run_experiment(wake, "jouk", ctrlset)
end
for c in ["BW600",
    "BW300",
    "BW120",
    "BW60",
    "BW30"]
    # run_adm_experiment(c)
    println(c)
end
for c in ["BW600",
    "BW300",
    "BW120",
    "BW60",
    "BW30"]
    run_admr_experiment(c)
end
for c in ["BW600",
    "BW300",
    "BW120",
    "BW60",
    "BW30"]
    run_jouk_experiment(c)
end
# wake = run_adm_experiment()
# run_admr_experiment()
# run_jouk_experiment()
