
include("mowitobase.jl")


function get_flow_data_yz(wake::VortexWake.Wake, xpos)
    D = wake.parameters.rotor.parameters.diameter
    y=-2*D:0.05*D:2*D
    # y=-200.:10.:200.
    z= 0:0.05*D:3*D #0.:10.:400.
    yg = repeat(reshape(y,1,:),length(z),1)
    zg = repeat(z,1,length(y))
    x = xpos
    points = [SVector{3}([x,yp,zp]) for yp in y for zp in z]
    uvec = velocity(points, wake)
    np = length(uvec)
    data = [reshape(yg, np) reshape(zg, np) [u[1] for u in uvec] [u[2] for u in uvec] [u[3] for u in uvec]] #reshape(uvec, np) reshape(w, np)]
    cols = ["y", "z", "u", "v","w"]
    return DataFrame(data,cols)
end

function get_flow_data_xy(wake::VortexWake.Wake, zpos)
    D = wake.parameters.rotor.parameters.diameter
    x=-2*D:0.05*D:10*D
    # y=-200.:10.:200.
    y= -2*D:0.05*D:3*D #0.:10.:400.
    xg = repeat(reshape(x,1,:),length(y),1)
    yg = repeat(y,1,length(x))
    z = zpos
    points = [SVector{3}([xp,yp,z]) for xp in x for yp in y]
    uvec = velocity(points, wake)
    np = length(uvec)
    data = [reshape(xg, np) reshape(yg, np) [u[1] for u in uvec] [u[2] for u in uvec] [u[3] for u in uvec]] 
    cols = ["x", "y", "u", "v","w"]
    return DataFrame(data,cols)
end

function run_slices(wake, yaw, name)
    num_steps = Int(30÷wake.parameters.time_step)
    m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    run_forward!(wake, m, u)

    xposlist = (1:1:7)
    for xpos in xposlist
        flowdata =get_flow_data_yz(wake, xpos)
        CSV.write(@sprintf("data/vwjl/%s_flowfield_Yaw%.0f_%02dD.csv",name,yaw,xpos), flowdata)
    end
    flowdata = get_flow_data_xy(wake, 1.3275)
    CSV.write(@sprintf("data/vwjl/%s_flowfield_Yaw%.0f_hubheight.csv",name,yaw), flowdata)
    return wake
end
function run_trace(wake, core_size, name)
    num_steps = Int(24÷wake.parameters.time_step)
    m = [SVector{2}([0.33, 30.]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    run_forward!(wake, m, u)
    pos = wake.states.positions
    data_x = [pos[b][1] for  b in eachindex(pos)]
    data_y = [pos[b][2] for  b in eachindex(pos)]
    data_z = [pos[b][3] for  b in eachindex(pos)]

    df = DataFrame([data_x data_y data_z],["x","y","z"])
    CSV.write(@sprintf("data/vwjl/%s_trace_s%.0f.csv",name,core_size*1e4), df)
    return wake
end
function run_adm_slices(yaw)
    return run_slices(construct_adm_wt_unit(40,16,100.,0.16), yaw, "ADM")
end
function run_admr_slices(yaw)
    return run_slices(construct_admr_wt_unit(40,16,100.), yaw, "ADMR")
end
function run_adm_ground_slices(yaw)
    return run_slices(construct_adm_ground_wt_unit(40,16,100.,0.16), yaw, "ADM_ground")
end
function run_admr_ground_slices(yaw)
    return run_slices(construct_admr_ground_wt_unit(40,16,100.), yaw, "ADMR_ground")
end
function run_jouk_slices(yaw)
    return run_slices(construct_joukowsky_wt_unit(100), yaw, "jouk")
end
function run_adm_slices_diff(delta)
    return run_slices(construct_adm_wt_unit(delta), 0, @sprintf("ADM_d%03d",delta))
end
run_slices(construct_adm_wt_unit(40,16,0.), 30, "ADM")
run_slices(construct_admr_wt_unit(40,16,0.), 30, "ADMR")
run_slices(construct_joukowsky_wt_unit(100.), 30, "jouk")

# run_trace(construct_adm_wt_unit(40,16,0., 0.016), 0.0016, "ADM")
run_trace(construct_adm_wt_unit(40,16,0., 0.00016), 0.00016, "ADM")
run_trace(construct_adm_wt_unit(40,16,0., 0.0016), 0.0016, "ADM")
run_trace(construct_adm_wt_unit(40,16,0., 0.016), 0.016, "ADM")
run_trace(construct_adm_wt_unit(40,16,0., 0.16), 0.16, "ADM")

run_trace(construct_adm_wt_unit(40,16,100., 0.16), 0.16, "ADM_fig_yaw")
run_trace(construct_adm_wt_unit(40,16,100., 0.16), 0.16, "ADM_fig")
run_trace(construct_admr_wt_unit(40,16,100.), 0.16, "ADMR_fig")
run_trace(construct_joukowsky_wt_unit(100), 0.16, "ALM_fig")

# run_adm_slices_diff(1000)
# run_adm_slices_diff(500)
# run_adm_slices_diff(100)
# run_adm_slices_diff(50)
# run_adm_slices_diff(20)
# run_adm_slices_diff(10)
# run_adm_slices_diff(1)
# wake = construct_adm_wt_unit()

for y in [0,-30.,30] #,-30.]
    # run_adm_slices(y)
    # run_admr_slices(y)
    # run_adm_ground_slices(y)
    run_admr_ground_slices(y)
    # run_jouk_slices(y)
end
for d in [10,20,50,100,200,500]
    run_adm_slices_diff(d)
end
# wake = run_admr_slices(30.)
# plot_points(wake)n



wake = construct_adm_ground_wt_unit(40,16,10.)
yaw = 30
wake = run_slices(construct_adm_ground_wt_unit(40,16,100.,0.16), yaw, "ADM_ground")
wake = run_slices(construct_admr_ground_wt_unit(40,16,100.), yaw, "ADMR_ground")
wake_ng = run_slices(construct_admr_wt_unit(40,16,100.), yaw, "ADMR_ng")
@time wake = run_slices_helix(wake, "helix_B")
plot_points(wake)
# num_steps = Int(10÷wake.parameters.time_step)
# yaw = 0.
# m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
# u = generate_inflow(1., 0.)
# VortexWake.run_forward_helix!(wake, m, u)
plot_points(wake)
function run_slices_helix(wake,name)
    yaw = 0
    num_steps = Int(30÷wake.parameters.time_step)
    m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
    u = generate_inflow(1., 0.)
    VortexWake.run_forward_helix!(wake, m, u)

    xposlist = (1:1:7)
    for xpos in xposlist
        flowdata =get_flow_data_yz(wake, xpos)
        CSV.write(@sprintf("data/vwjl/%s_flowfield_Yaw%.0f_%02dD.csv",name,yaw,xpos), flowdata)
    end
    flowdata = get_flow_data_xy(wake, 1.3275)
    CSV.write(@sprintf("data/vwjl/%s_flowfield_Yaw%.0f_hubheight.csv",name,yaw), flowdata)

    pos = wake.states.positions
    data_x = [pos[b][1] for  b in eachindex(pos)]
    data_y = [pos[b][2] for  b in eachindex(pos)]
    data_z = [pos[b][3] for  b in eachindex(pos)]

    df = DataFrame([data_x data_y data_z],["x","y","z"])
    CSV.write(@sprintf("data/vwjl/%s_trace.csv",name), df)
    return wake
end



num_steps = 60
m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
u = generate_inflow(1., 0.)
@time run_forward!(wake, m, u)
p = wake.parameters
range(start=0,step=2π/p.num_elements, length=p.num_elements)


num_steps = 60
m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
u = generate_inflow(1., 0.)
run_forward!(waked, m, u)


wake = construct_adm_wt_unit(40,16,100.)
yaw = 10
num_steps = Int(30÷wake.parameters.time_step)
m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
u = generate_inflow(1., 0.)
VortexWake.run_forward_waked!(wake, m, u)

plot_points(wake)

thrust_coefficient(wake.parameters.rotor) * (1-0.33)^2

wake = construct_adm_wt_unit(40,16,100.)
yaw = 0
num_steps = Int(30÷wake.parameters.time_step)
m = [SVector{2}([0.33, yaw]) for c in 1:num_steps]
u = generate_inflow(1., 0.)
VortexWake.run_forward!(wake, m, u)

p = wake.parameters
xe = p.rotor.parameters.radius .* cos.(range(start=π/p.num_elements,step=2π/p.num_elements, length=p.num_elements))
sum(xe.<-0.1)

plot(xe[xe.<-0.1])