include("setup.jl")

const mowito_diameter = 0.58
const inflow_wt = 6.8

function construct_momentum_rotor_unit(h=0.3)
    parameters = MomentumRotorParameters(diameter=1, time_step=h, air_density=1)
     controls = MomentumRotorControls(induction=0.33, velocity=1.)
     states = MomentumRotorStates(azimuth=0.)
     return MomentumRotor(parameters, controls, states)
 end

function construct_fixed_tower_wt_unit(position)
    params = FixedTowerParameters(position, 1.3275, 0., )
    states = FixedTowerStates()
    controls = FixedTowerControls(0., 0.)
    return FixedTower(params, states, controls)
end

function construct_adm_wt_unit(rings=40,elements=16,delta=0.,sigma=0.16)
    rotor = construct_momentum_rotor_unit()
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = ADMWakeParameters(rotor=rotor, tower=tower,
        num_rings=rings, num_elements=elements,
        rotor_points=rotor_disc_points() ,
        time_step=0.3,
         initial_core_size=sigma, delta=delta)
    states = initialise_states(params)
    controls = ADMWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)))
    return ADMWake(params, states, controls)
end
function construct_adm_ground_wt_unit(rings=40,elements=16,delta=0.,sigma=0.16)
    rotor = construct_momentum_rotor_unit()
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = ADMWakeParameters(rotor=rotor, tower=tower,
        num_rings=rings, num_elements=elements,
        rotor_points=rotor_disc_points() ,
        time_step=0.3,
         initial_core_size=sigma, delta=delta,mirror_ground=true)
    states = initialise_states(params)
    controls = ADMWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)))
    return ADMWake(params, states, controls)
end

function construct_admr_wt_unit(rings=40,elements=16,delta=0.)
    rotor = construct_momentum_rotor_unit()
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = ADMRWakeParameters(rotor=rotor, tower=tower,
        num_rings=rings, num_elements=elements,
        rotor_points=rotor_disc_points(),
        time_step=0.3,
         initial_core_size=0.16,delta=delta)
    states = initialise_states(params)
    controls = ADMRWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)), omega=0.)
    return ADMRWake(params, states, controls)
end
function construct_admr_ground_wt_unit(rings=40,elements=16,delta=0.)
    rotor = construct_momentum_rotor_unit()
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = ADMRWakeParameters(rotor=rotor, tower=tower,
        num_rings=rings, num_elements=elements,
        rotor_points=rotor_disc_points(),
        time_step=0.3,
         initial_core_size=0.16,delta=delta,mirror_ground=true)
    states = initialise_states(params)
    controls = ADMRWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)), omega=0.)
    return ADMRWake(params, states, controls)
end
function construct_joukowsky_wt_unit(delta=0.)
    rotor = construct_momentum_rotor_unit(0.033)
    tower = construct_fixed_tower_wt_unit(@SVector zeros(3))
    params = JoukowskyWakeParameters(rotor=rotor, tower=tower,
        num_rings=360, num_blades=3,
        rotor_points=rotor_disc_points(),
        time_step=0.033,
         initial_core_size=  0.16,delta=delta)
    states = initialise_states(params)
    controls = JoukowskyWakeControls(thrust = 0., inflow = SVector{3}(zeros(3)), omega=1.)
    return JoukowskyWake(params, states, controls)
end